      SUBROUTINE SPLINE (N, X, Y, B, C, D)
      INTEGER N
      REAL X(N), Y(N), B(N), C(N), D(N)
C
C  THE COEFFICIENTS B(I), C(I), AND D(I), I=1,2,...,N ARE COMPUTED
C  FOR A CUBIC INTERPOLATING SPLINE
C
C    S(X) = Y(I) + B(I)*(X-X(I)) + C(I)*(X-X(I))**2 + D(I)*(X-X(I))**3
C
C    FOR  X(I) .LE. X .LE. X(I+1)
C
C  INPUT..
C
C    N = THE NUMBER OF DATA POINTS OR KNOTS (N.GE.2)
C    X = THE ABSCISSAS OF THE KNOTS IN STRICTLY INCREASING ORDER
C    Y = THE ORDINATES OF THE KNOTS
C
C  OUTPUT..
C
C    B, C, D  = ARRAYS OF SPLINE COEFFICIENTS AS DEFINED ABOVE.
C
C  USING  P  TO DENOTE DIFFERENTIATION,
C
C    Y(I) = S(X(I))
C    B(I) = SP(X(I))
C    C(I) = SPP(X(I))/2
C    D(I) = SPPP(X(I))/6  (DERIVATIVE FROM THE RIGHT)
C
C  THE ACCOMPANYING FUNCTION SUBPROGRAM  SEVAL  CAN BE USED
C  TO EVALUATE THE SPLINE.
C
C
      INTEGER NM1, IB, I
      REAL T
C
      NM1 = N-1
      IF ( N .LT. 2 ) RETURN
      IF ( N .LT. 3 ) GO TO 50
C
C  SET UP TRIDIAGONAL SYSTEM
C
C  B = DIAGONAL, D = OFFDIAGONAL, C = RIGHT HAND SIDE.
C
      D(1) = X(2) - X(1)
      C(2) = (Y(2) - Y(1))/D(1)
      DO 10 I = 2, NM1
         D(I) = X(I+1) - X(I)
         B(I) = 2.*(D(I-1) + D(I))
         C(I+1) = (Y(I+1) - Y(I))/D(I)
         C(I) = C(I+1) - C(I)
   10 CONTINUE
C
C  END CONDITIONS.  THIRD DERIVATIVES AT  X(1)  AND  X(N)
C  OBTAINED FROM DIVIDED DIFFERENCES
C
      B(1) = -D(1)
      B(N) = -D(N-1)
      C(1) = 0.
      C(N) = 0.
      IF ( N .EQ. 3 ) GO TO 15
      C(1) = C(3)/(X(4)-X(2)) - C(2)/(X(3)-X(1))
      C(N) = C(N-1)/(X(N)-X(N-2)) - C(N-2)/(X(N-1)-X(N-3))
      C(1) = C(1)*D(1)**2/(X(4)-X(1))
      C(N) = -C(N)*D(N-1)**2/(X(N)-X(N-3))
C
C  FORWARD ELIMINATION
C
   15 DO 20 I = 2, N
         T = D(I-1)/B(I-1)
         B(I) = B(I) - T*D(I-1)
         C(I) = C(I) - T*C(I-1)
   20 CONTINUE
C
C  BACK SUBSTITUTION
C
      C(N) = C(N)/B(N)
      DO 30 IB = 1, NM1
         I = N-IB
         C(I) = (C(I) - D(I)*C(I+1))/B(I)
   30 CONTINUE
C
C  C(I) IS NOW THE SIGMA(I) OF THE TEXT
C
C  COMPUTE POLYNOMIAL COEFFICIENTS
C
      B(N) = (Y(N) - Y(NM1))/D(NM1) + D(NM1)*(C(NM1) + 2.*C(N))
      DO 40 I = 1, NM1
         B(I) = (Y(I+1) - Y(I))/D(I) - D(I)*(C(I+1) + 2.*C(I))
         D(I) = (C(I+1) - C(I))/D(I)
         C(I) = 3.*C(I)
   40 CONTINUE
      C(N) = 3.*C(N)
      D(N) = D(N-1)
      RETURN
C
   50 B(1) = (Y(2)-Y(1))/(X(2)-X(1))
      C(1) = 0.
      D(1) = 0.
      B(2) = B(1)
      C(2) = 0.
      D(2) = 0.
      RETURN
      END
      SUBROUTINE SPLMI (N, X, Y, B, C, D)
      INTEGER N
      REAL X(N), Y(N), B(N), C(N), D(N),DELTA(50),H(50),AL(50),BE(50)
C
C  THE COEFFICIENTS B(I), C(I), AND D(I), I=1,2,...,N ARE COMPUTED
C  FOR A MONOTONICALLY VARYING CUBIC INTERPOLATING SPLINE
C
C    S(X) = Y(I) + B(I)*(X-X(I)) + C(I)*(X-X(I))**2 + D(I)*(X-X(I))**3
C
C    FOR  X(I) .LE. X .LE. X(I+1)
C    WITH Y(I+1).GE.Y(I) (ALL I) OR Y(I+1).LT.Y(I) (ALL I)
C
C  INPUT..
C
C    N = THE NUMBER OF DATA POINTS OR KNOTS (N.GE.2)
C    X = THE ABSCISSAS OF THE KNOTS IN STRICTLY INCREASING ORDER
C    Y = THE ORDINATES OF THE KNOTS
C
C  OUTPUT..
C
C    B, C, D  = ARRAYS OF SPLINE COEFFICIENTS AS DEFINED ABOVE.
C
C*************************************************************************
C
C THEORY FROM 'MONOTONE PIECEWISE CUBIC INTERPOLATION',
C BY F.N. FRITSCH AND R.E. CARLSON IN SIAM J.NUMER.ANAL.,V.17,P.238
C
C*************************************************************************
C
C    Y(I) = S(X(I))
C    B(I) = SP(X(I))
C    C(I) = SPP(X(I))/2
C    D(I) = SPPP(X(I))/6  (DERIVATIVE FROM THE RIGHT)
C
C  THE ACCOMPANYING FUNCTION SUBPROGRAM  SEVAL  CAN BE USED
C  TO EVALUATE THE SPLINE.
C
C
      NM1 = N-1
      IF ( N .LT. 2 ) RETURN
      IF ( N .LT. 3 ) GO TO 100
C
C CALCULATE THE H(I) AND DELTA(I)
C
      DO 10 I=1,NM1
      AL(I)=0.
      BE(I)=0.
      H(I)=X(I+1)-X(I)
   10 DELTA(I)=(Y(I+1)-Y(I))/H(I)
C
C CALCULATE FIRST VALUES FOR AL AND BE BY 3-POINT DIFFERENCE
C
      IF(DELTA(1).EQ.0) GOTO 15
      AL(1)=((H(1)+H(2))**2*Y(2)-H(1)**2*Y(3)-H(2)*(2.*H(1)+H(2))
     1        *Y(1))/(H(2)*(H(1)+H(2))*(Y(2)-Y(1)))
   15 DO 20 I=2,NM1
      IF(DELTA(I).EQ.0) GOTO 20
      AL(I)=(H(I-1)**2*Y(I+1)+(H(I)**2-H(I-1)**2)*Y(I)-H(I)**2*
     1        Y(I-1))/(H(I-1)*(H(I)+H(I-1))*(Y(I+1)-Y(I)))
   20 CONTINUE
C
      NM2=N-2
      DO 30 I=1,NM2
      IF(DELTA(I).EQ.0.) GOTO 30
      BE(I)=(H(I)**2*Y(I+2)+(H(I+1)**2-H(I)**2)*Y(I+1)-H(I+1)**2*
     1        Y(I))/(H(I+1)*(H(I)+H(I+1))*(Y(I+1)-Y(I)))
   30 CONTINUE
C
      IF(DELTA(N-1).EQ.0.) GOTO 35
      BE(N-1)=(H(N-2)*(2.*H(N-1)+H(N-2))*Y(N)-(H(N-1)+H(N-2))**2
     1        *Y(N-1)+H(N-1)**2*Y(N-2))/(H(N-2)*(H(N-1)+H(N-2))
     2        *(Y(N)-Y(N-1)))
c
C CORRECT VALUES FOR AL AND BE
C
   35 DO 40 I=1,NM1
      IF(AL(I)+BE(I).LE.2.) GOTO 40
      IF(2.*AL(I)+BE(I).LE.3.) GOTO 40
      IF(AL(I)+2.*BE(I).LE.3.) GOTO 40
      PHI=AL(I)-(2.*AL(I)+BE(I)-3.)**2/(AL(I)+BE(I)-2.)/3.
      IF(PHI.GE.0.) GOTO 40
      TI=3./SQRT(AL(I)**2+BE(I)**2)
      AL(I)=TI*AL(I)
      BE(I)=TI*BE(I)
   40 CONTINUE
C
C CALCULATE SPLINE COEFFICIENTS
C
      DO 50 I=1,NM1
      D(I)=(AL(I)+BE(I)-2.)*DELTA(I)/H(I)**2
      C(I)=(3.-2.*AL(I)-BE(I))*DELTA(I)/H(I)
      B(I)=AL(I)*DELTA(I)
      IF(B(I)*DELTA(I).GE.0.) GOTO 50
        write(*,*)b(i),delta(i)
      B(I)=DELTA(I)
      C(I)=0.
      D(I)=0.
   50 CONTINUE
      B(N)=BE(N-1)*DELTA(N-1)
      C(N)=0.
      D(N)=0.
      RETURN
C
  100 B(1) = (Y(2)-Y(1))/(X(2)-X(1))
      C(1) = 0.
      D(1) = 0.
      B(2) = B(1)
      C(2) = 0.
      D(2) = 0.
      RETURN
      END
      REAL FUNCTION SEVAL(N, U, X, Y, B, C, D)
      INTEGER N
      REAL  U, X(N), Y(N), B(N), C(N), D(N)
C
C  THIS SUBROUTINE EVALUATES THE CUBIC SPLINE FUNCTION
C
C    SEVAL = Y(I) + B(I)*(U-X(I)) + C(I)*(U-X(I))**2 + D(I)*(U-X(I))**3
C
C    WHERE  X(I) .LT. U .LT. X(I+1), USING HORNER'S RULE
C
C  IF  U .LT. X(1) THEN  I = 1  IS USED.
C  IF  U .GE. X(N) THEN  I = N  IS USED.
C
C  INPUT..
C
C    N = THE NUMBER OF DATA POINTS
C    U = THE ABSCISSA AT WHICH THE SPLINE IS TO BE EVALUATED
C    X,Y = THE ARRAYS OF DATA ABSCISSAS AND ORDINATES
C    B,C,D = ARRAYS OF SPLINE COEFFICIENTS COMPUTED BY SPLINE
C
C  IF  U  IS NOT IN THE SAME INTERVAL AS THE PREVIOUS CALL, THEN A
C  BINARY SEARCH IS PERFORMED TO DETERMINE THE PROPER INTERVAL.
C
      INTEGER I, J, K
      REAL DX
      DATA I/1/
      IF ( I .GE. N ) I = 1
      IF ( U .LT. X(I) ) GO TO 10
      IF ( U .LE. X(I+1) ) GO TO 30
C
C  BINARY SEARCH
C
   10 I = 1
      J = N+1
   20 K = (I+J)/2
      IF ( U .LT. X(K) ) J = K
      IF ( U .GE. X(K) ) I = K
      IF ( J .GT. I+1 ) GO TO 20
C
C  EVALUATE SPLINE
C
   30 DX = U - X(I)
      SEVAL = Y(I) + DX*(B(I) + DX*(C(I) + DX*D(I)))
      RETURN
      END
